function [] = runExtension(RestartLevel)

syms x xh y yh a ah b bh;
X = [x xh y yh a ah b bh];

syms g1 gh1 g2 gh2 g3 gh3 g4 gh4 g5 gh5 g6 gh6 g7 gh7 g8 gh8;
G = [g1 gh1 g2 gh2 g3 gh3 g4 gh4 g5 gh5 g6 gh6 g7 gh7 g8 gh8];

if nargin == 0 || RestartLevel < 2
  % begin a new solve
  A = rand(8,8)*2 - 1 + 1i*(rand(8,8)*2 - 1);
  CreateStart(A,8,8,'slices_A');

  % solve first system
  solveFirst(A,G,X);

  % solve k = 2,..,8 systems (degenerate test when k = 9)
  for k = 2:9
    solveSystem(k,A,G,X);
  end;
else
  [~,A] = ReadInStart(8,'slices_A');

  if RestartLevel <= 8
    disp(' ');
    disp(['Solving system ',num2str(RestartLevel)]);
    eval(['!mpirun bertini input_',num2str(RestartLevel),' start_',num2str(RestartLevel),' > /dev/null']);

    for k = RestartLevel+1:9 % degenerate test when k = 9
      solveSystem(k,A,G,X);
    end;
  else % only perform degenerate test
    solveSystem(9,A,G,X);
  end;
end;

return;

function [] = solveSystem(k,A,G,X)

% perform degenerate test on finite_solutions
degenerateTest(k-1,G);

if k > 8
  return;
end;

[n,P] = ReadInStart(8+2*(k-1),['solutions_',num2str(k-1)]);
disp(['Solutions from system ',num2str(k-1),': ',num2str(n)]);
disp(' ');

% setup new slices
S = 2*rand(3,8) - 1 + 1i*(2*rand(3,8) - 1);

% move slices 
for j = 1:3
  disp(['Moving slices']);
  F = fopen('input_bottom','w');
  fprintf(F,'pathvariable t;\nparameter s;\ns = t;\n');
  % setup variables and main functions
  for l = 1:k-1
    fprintf(F,'variable_group %s,%s;\n',char(G(2*(l-1)+1)),char(G(2*(l-1)+2)));
    fprintf(F,'function G%d1,G%d2,G%d3;\n',l,l,l);
    printPoly3(l,F);
  end;
  for l = k:8
    fprintf(F,'function l%d;\n',l);
  end;
  % setup constants
  printConstants(k-1,A,F);

  % setup moving slice
  fprintf(F,'constant s1,s2,s3,s4,s5,s6,s7,s8;\n');
  for l = 1:8
    fprintf(F,'s%d = %s + I*%s;\n',l,char(vpa(real(S(j,l)))),char(vpa(imag(S(j,l)))));
  end;
  fprintf(F,'l%d = ',k);
  for l = 1:8
    fprintf(F,'(r%d%d*s + (1-s)*s%d)*%s + ',k,l,l,char(X(l)));
  end;
  fprintf(F,' - 1;\n');

  % setup fixed linears
  printLinear(k,A,F,X);
  fprintf(F,'END;\n');
  fclose(F);

  disp(['Moving linear system ',num2str(l),': ',num2str(n),' paths']);
  eval(['!rm -rf input; cat input_top input_bottom > input; mpirun bertini input solutions_',num2str(k-1),' > /dev/null']);

  [n1,P1] = ReadInStart(8+2*(k-1),'finite_solutions');
 
  % Check for errors
  if n ~= n1
    disp(['ERROR: Failure in moving a linear slice!']);
    return;
  end;

  % update P
  P = [P;P1];
end;

% setup slices for gamma
T = 2*rand(4,2) - 1 + 1i*(2*rand(4,2) - 1);

% solve for gamma
G13 = transpose(T([1,3],:)\ones(2,1));
G14 = transpose(T([1,4],:)\ones(2,1));
G23 = transpose(T([2,3],:)\ones(2,1));
G24 = transpose(T([2,4],:)\ones(2,1));

% setup new start points
Q = zeros(8*n,8+2*k);
for j = 1:n
  Q(j,:) = [P(j,:) G23];
  Q(n+j,:) = [P(j,:) G24];
  Q(2*n+j,:) = [P(n+j,:) G23];
  Q(3*n+j,:) = [P(n+j,:) G24];
  Q(4*n+j,:) = [P(2*n+j,:) G13];
  Q(5*n+j,:) = [P(2*n+j,:) G14];
  Q(6*n+j,:) = [P(3*n+j,:) G13];
  Q(7*n+j,:) = [P(3*n+j,:) G14];
end;

% print to file
CreateStart(Q,8*n,8+2*k,['start_',num2str(k)]);

% create system
F = fopen('input_bottom','w');
fprintf(F,'pathvariable t;\nparameter s;\ns = t;\n');
% setup variables and main functions
for l = 1:k-1
  fprintf(F,'variable_group %s,%s;\n',char(G(2*(l-1)+1)),char(G(2*(l-1)+2)));
  fprintf(F,'function G%d1,G%d2,G%d3;\n',l,l,l);
  printPoly3(l,F);
end;
for l = k+1:8
  fprintf(F,'function l%d;\n',l);
end;
% setup constants
printConstants(k-1,A,F);
% setup fixed linears
printLinear(k,A,F,X);

% setup moving polynomials
fprintf(F,'variable_group %s,%s;\n',char(G(2*k-1)),char(G(2*k)));
fprintf(F,'function H1,H2,H3;\n');
printPoly3(k,F);
for j = 1:4
  fprintf(F,'constant t%d1,t%d2;\n',j,j);
  for l = 1:2
    fprintf(F,'t%d%d = %s + I*%s;\n',j,l,char(vpa(real(T(j,l)))),char(vpa(imag(T(j,l)))));
  end;
end;
for j = 1:3
  fprintf(F,'constant s%d1,s%d2,s%d3,s%d4,s%d5,s%d6,s%d7,s%d8;\n',j,j,j,j,j,j,j,j);
  for l = 1:8
    fprintf(F,'s%d%d = %s + I*%s;\n',j,l,char(vpa(real(S(j,l)))),char(vpa(imag(S(j,l)))));
  end;
end;
fprintf(F,'H1 = (');
for l = 1:8
  fprintf(F,'r%d%d*%s + ',k,l,char(X(l)));
end;
fprintf(F,' - 1)*(');
for l = 1:8
  fprintf(F,'s1%d*%s + ',l,char(X(l)));
end;
fprintf(F,' - 1)*(t11*%s + t12*%s - 1)*s + (1-s)*G%d1;\n',char(G(2*k-1)),char(G(2*k)),k);
fprintf(F,'H2 = (');
for l = 1:8
  fprintf(F,'s2%d*%s + ',l,char(X(l)));
end;
fprintf(F,' - 1)*(');
for l = 1:8
  fprintf(F,'s3%d*%s + ',l,char(X(l)));
end;
fprintf(F,' - 1)*(t21*%s + t22*%s - 1)*s + (1-s)*G%d2;\n',char(G(2*k-1)),char(G(2*k)),k);
fprintf(F,'H3 = (t31*%s + t32*%s - 1)*(t41*%s + t42*%s - 1)*s + (1-s)*G%d3;\n',char(G(2*k-1)),char(G(2*k)),char(G(2*k-1)),char(G(2*k)),k);
fprintf(F,'END;\n');
fclose(F);

disp(' ');
disp(['Solving system ',num2str(k),': ',num2str(8*n),' paths']);
eval(['!rm -rf input_',num2str(k),'; cat input_top input_bottom > input_',num2str(k),'; mpirun bertini input_',num2str(k),' start_',num2str(k),' > /dev/null']);

return;

function [] = solveFirst(A,G,X)

% open file
F = fopen('input_1_bottom','w');

% setup variables
fprintf(F, 'variable_group %s,%s;\n',char(G(1)),char(G(2)));

% setup constants
printConstants(1,A,F);

% setup polynomials
fprintf(F, 'function G11,G12,G13,l2,l3,l4,l5,l6,l7,l8;\n');
printPoly3(1,F);
printLinear(1,A,F,X);
fprintf(F,'END;\n');
fclose(F);

disp(['Solving system 1']);
eval(['!rm -rf input_1; cat input_1_top input_1_bottom > input_1; mpirun bertini input_1 > /dev/null']);

return;

function [] = printConstants(k,A,F)

c = 'constant ';
for j = k+1:8
  for l = 1:8
    c = [c,'r',num2str(j),num2str(l)];
    if j == 8 && l == 8
      c = [c,';'];
    else
      c = [c,','];
    end;
  end;
end;
fprintf(F,'%s\n',c);

for j = k+1:8
  for l = 1:8
    fprintf(F,'r%d%d = %s + I*%s;\n',j,l,char(vpa(real(A(j,l)))),char(vpa(imag(A(j,l)))));
  end;
 end;

return;

function [] = printLinear(k,A,F,X)

for j = k+1:8
  c = ['l',num2str(j),' = '];
  for l = 1:8 
    c = [c,'r',num2str(j),num2str(l),'*',char(X(l))];
    if l < 8
      c = [c,' + '];
    else
      c = [c,' - 1;'];
    end;
  end;
  fprintf(F,'%s\n',c);
end;

return;

function [] = printPoly3(k,F)

fprintf(F,'G%d1 = (nh - dh%d*x)*g%d + (n - d%d*xh)*gh%d + (d%d*(ah - xh) + dh%d*(a - x) - d%d*dh%d);\n',k,k,k,k,k,k,k,k,k);
fprintf(F,'G%d2 = (mh - dh%d*y)*g%d + (m - d%d*yh)*gh%d + (d%d*(bh - yh) + dh%d*(b - y) - d%d*dh%d);\n',k,k,k,k,k,k,k,k,k);
fprintf(F,'G%d3 = g%d*gh%d + g%d + gh%d;\n',k,k,k,k,k);

return;

function [] = degenerateTest(k,G)

tol = 1e-9;

% move finite_solutions
eval(['!mv finite_solutions solutions_',num2str(k)]);

% setup variable list
F = fopen('input_degenerate_middle','w');
for j = 1:k
  fprintf(F,'variable_group %s,%s;\n',char(G(2*(j-1)+1)),char(G(2*(j-1)+2)));
end;
fclose(F);

disp(['Running degenerate test ',num2str(k)]);
eval(['!rm -rf input_degenerate_',num2str(k),'; cat input_degenerate_top input_degenerate_middle input_degenerate_bottom > input_degenerate_',num2str(k),'; bertini input_degenerate_',num2str(k),' solutions_',num2str(k),' > /dev/null']);

[n,F] = ReadInStart(12,'function');
Good = n;

if min(min(abs(F))) < tol
  % we need to read in the file and remove the degenerate ones
  [n,P] = ReadInStart(8+2*k,['solutions_',num2str(k)]);
  G = ones(1,n);
  for j = 1:n
    if min(abs(F(j,:))) < tol
      G(j) = 0;
    end;
  end;
  PG = P(G > 0,:);
  Good = size(PG,1);
  CreateStart(PG,Good,8+2*k,['solutions_',num2str(k)]);
end;

disp(['Degenerate test ',num2str(k),': ',num2str(Good),' of ',num2str(n),' are nondegenerate']);

return;



